# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from datetime import timedelta

from django.core.urlresolvers import reverse
from django.test import TransactionTestCase
from django.utils.timezone import now
from django.utils.translation import override

from aldryn_newsblog.feeds import LatestArticlesFeed, TagFeed, CategoryFeed

from . import NewsBlogTestsMixin


class TestFeeds(NewsBlogTestsMixin, TransactionTestCase):

    def test_latest_feeds(self):
        article = self.create_article()
        future_article = self.create_article(
            publishing_date=now() + timedelta(days=3),
            is_published=True,
        )
        url = reverse(
            '{0}:article-list-feed'.format(self.app_config.namespace)
        )
        self.request = self.get_request('en', url)
        self.request.current_page = self.page
        feed = LatestArticlesFeed()(self.request)

        self.assertContains(feed, article.title)
        self.assertNotContains(feed, future_article.title)

    def test_tag_feed(self):
        articles = self.create_tagged_articles()

        url = reverse(
            '{0}:article-list-by-tag-feed'.format(self.app_config.namespace),
            args=['tag1']
        )
        self.request = self.get_request('en', url)
        if getattr(self.request, 'current_page', None) is None:
            self.request.current_page = self.page
        feed = TagFeed()(self.request, 'tag1')

        for article in articles['tag1']:
            self.assertContains(feed, article.title)
        for different_tag_article in articles['tag2']:
            self.assertNotContains(feed, different_tag_article.title)

    def test_category_feed(self):
        lang = self.category1.get_current_language()
        with override(lang):
            article = self.create_article()
            article.categories.add(self.category1)
            different_category_article = self.create_article()
            different_category_article.categories.add(self.category2)
            url = reverse(
                '{0}:article-list-by-category-feed'.format(
                    self.app_config.namespace),
                args=[self.category1.slug]
            )
            self.request = self.get_request(lang, url)
            if getattr(self.request, 'current_page', None) is None:
                self.request.current_page = self.page

            feed = CategoryFeed()(self.request, self.category1.slug)

            self.assertContains(feed, article.title)
            self.assertNotContains(feed, different_category_article.title)



# -*- coding: utf-8 -*-

from django.test import TestCase
from django.test.client import RequestFactory

from cms.api import add_plugin
from cms.models import Placeholder
from cms.plugin_rendering import ContentRenderer

from myapp.cms_plugins import MyPlugin
from myapp.models import MyappPlugin

class LightsliderPluginTests(TestCase):
    def test_plugin_context(self):
        placeholder = Placeholder.objects.create(slot='test')
        model_instance = add_plugin(
            placeholder,
            MyPlugin,
            'en',
        )
        plugin_instance = model_instance.get_plugin_class_instance()
        context = plugin_instance.render({}, model_instance, None)
        self.assertIn('key', context)
        self.assertEqual(context['key'], 'value')

    def test_plugin_html(self):
        placeholder = Placeholder.objects.create(slot='test')
        model_instance = add_plugin(
            placeholder,
            MyPlugin,
            'en',
        )
        renderer = ContentRenderer(request=RequestFactory())
        html = renderer.render_plugin(model_instance, {})
        self.assertEqual(html, '<strong>Test</strong>')


# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from aldryn_newsblog.models import Article

from . import NewsBlogTestCase


class TestManagers(NewsBlogTestCase):

    def test_published_articles_filtering(self):
        for i in range(5):
            self.create_article()
        unpublised_article = Article.objects.first()
        unpublised_article.is_published = False
        unpublised_article.save()
        self.assertEqual(Article.objects.published().count(), 4)
        self.assertNotIn(unpublised_article, Article.objects.published())

    # TODO: Should also test for publishing_date
    def test_view_article_not_published(self):
        article = self.create_article(is_published=False)
        article_url = article.get_absolute_url()
        response = self.client.get(article_url)
        self.assertEqual(response.status_code, 404)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os

from datetime import datetime, date
from operator import itemgetter
from random import randint

from django.conf import settings
from django.core.files import File as DjangoFile
from django.core.urlresolvers import reverse, NoReverseMatch
from django.utils.timezone import now
from django.utils.translation import override

from aldryn_newsblog.models import Article, NewsBlogConfig
from aldryn_newsblog.search_indexes import ArticleIndex
from cms.utils.i18n import get_current_language, force_language
from easy_thumbnails.files import get_thumbnailer
from filer.models.imagemodels import Image
from parler.tests.utils import override_parler_settings
from parler.utils.conf import add_default_language_settings
from parler.utils.context import switch_language, smart_override

from . import NewsBlogTestCase, TESTS_STATIC_ROOT

FEATURED_IMAGE_PATH = os.path.join(TESTS_STATIC_ROOT, 'featured_image.jpg')

PARLER_LANGUAGES_HIDE = {
    1: [
        {
            'code': u'en',
            'fallbacks': [u'de'],
            'hide_untranslated': True
        },
        {
            'code': u'de',
            'fallbacks': [u'en'],
            'hide_untranslated': True
        },
        {
            'code': u'fr',
            'fallbacks': [u'en'],
            'hide_untranslated': True
        },
    ],
    'default': {
        'hide_untranslated': True,
        'fallbacks': [],
    }
}

PARLER_LANGUAGES_SHOW = {
    1: [
        {
            'code': u'en',
            'fallbacks': [u'de'],
            'hide_untranslated': False
        },
        {
            'code': u'de',
            'fallbacks': [u'en'],
            'hide_untranslated': False
        },
        {
            'code': u'fr',
            'fallbacks': [u'en'],
            'hide_untranslated': False
        },
    ],
    'default': {
        'hide_untranslated': False,
        'fallbacks': [],
    }
}


class TestViews(NewsBlogTestCase):

    def test_articles_list(self):
        namespace = self.app_config.namespace
        articles = [self.create_article() for _ in range(11)]
        unpublished_article = articles[0]
        unpublished_article.is_published = False
        unpublished_article.save()
        response = self.client.get(
            reverse('{0}:article-list'.format(namespace)))
        for article in articles[1:]:
            self.assertContains(response, article.title)
        self.assertNotContains(response, unpublished_article.title)

    def test_articles_list_exclude_featured(self):
        namespace = self.app_config.namespace
        # configure app config
        exclude_count = 2
        self.app_config.exclude_featured = exclude_count
        self.app_config.paginate_by = 2
        self.app_config.save()
        # set up articles
        articles = []
        featured_articles = []
        for idx in range(6):
            if idx % 2:
                featured_articles.append(self.create_article(is_featured=True))
            else:
                articles.append(self.create_article())
        # imitate ordering by publish date DESC
        articles.reverse()
        featured_articles.reverse()
        # prepare urls
        list_base_url = reverse('{0}:article-list'.format(namespace))
        page_url_template = '{0}?page={1}'
        response_page_1 = self.client.get(list_base_url)
        response_page_2 = self.client.get(
            page_url_template.format(list_base_url, 2))

        # page 1
        # ensure that first two not featured articles are present on first page
        for article in articles[:2]:
            self.assertContains(response_page_1, article.title)
        # Ensure no featured articles are present on first page.
        for featured_article in featured_articles[:2]:
            self.assertNotContains(response_page_1, featured_article.title)

        # page 2
        # check that not excluded featured article is present on second page
        for featured_article in featured_articles[2:]:
            self.assertContains(response_page_2, featured_article.title)
        # ensure that third not featured article is present in the response
        for article in articles[2:]:
            self.assertContains(response_page_2, article.title)

    def test_articles_list_pagination(self):
        namespace = self.app_config.namespace
        paginate_by = self.app_config.paginate_by
        articles = [self.create_article(
            app_config=self.app_config,
            publishing_date=datetime(2000 - i, 1, 1, 1, 1)
        ) for i in range(paginate_by + 5)]

        response = self.client.get(
            reverse('{0}:article-list'.format(namespace)))
        for article in articles[:paginate_by]:
            self.assertContains(response, article.title)
        for article in articles[paginate_by:]:
            self.assertNotContains(response, article.title)

        response = self.client.get(
            reverse('{0}:article-list'.format(namespace)) + '?page=2')
        for article in articles[:paginate_by]:
            self.assertNotContains(response, article.title)
        for article in articles[paginate_by:]:
            self.assertContains(response, article.title)

    def test_articles_by_author(self):
        author1, author2 = self.create_person(), self.create_person()
        for author in (author1, author2):
            articles = [
                self.create_article(author=author) for _ in range(11)]
            response = self.client.get(reverse(
                'aldryn_newsblog:article-list-by-author',
                kwargs={'author': author.slug}))
            for article in articles:
                self.assertContains(response, article.title)

    def test_articles_by_unknown_author(self):
        response = self.client.get(reverse(
            'aldryn_newsblog:article-list-by-author',
            kwargs={'author': 'unknown'}))
        self.assertEqual(response.status_code, 404)

    def test_articles_by_category(self):
        """
        Tests that we can find articles by their categories, in ANY of the
        languages they are translated to.
        """
        LANGUAGES = add_default_language_settings(PARLER_LANGUAGES_HIDE)
        with override_parler_settings(PARLER_LANGUAGES=LANGUAGES):
            author = self.create_person()
            for category in (self.category1, self.category2):
                articles = []
                code = "{0}-".format(self.language)
                for _ in range(11):
                    article = Article.objects.create(
                        title=self.rand_str(),
                        slug=self.rand_str(prefix=code),
                        app_config=self.app_config,
                        author=author,
                        owner=author.user,
                        publishing_date=now(),
                        is_published=True,
                    )
                    # Make sure there are translations in place for the
                    # articles.
                    for language, _ in settings.LANGUAGES[1:]:
                        with switch_language(article, language):
                            code = "{0}-".format(language)
                            article.title = self.rand_str(prefix=code)
                            article.save()

                    article.categories.add(category)
                    articles.append(article)

                for language, _ in settings.LANGUAGES:
                    with switch_language(category, language):
                        url = reverse(
                            'aldryn_newsblog:article-list-by-category',
                            kwargs={'category': category.slug})
                        response = self.client.get(url)
                    for article in articles:
                        if language in article.get_available_languages():
                            article.set_current_language(language)
                            self.assertContains(response, article.title)
                        else:
                            article.set_current_language(language)
                            self.assertNotContains(response, article.title)

    def test_articles_by_unknown_category(self):
        response = self.client.get(reverse(
            'aldryn_newsblog:article-list-by-category',
            kwargs={'category': 'unknown'}))
        self.assertEqual(response.status_code, 404)


class TestTemplatePrefixes(NewsBlogTestCase):

    def setUp(self):
        super(TestTemplatePrefixes, self).setUp()
        self.app_config.template_prefix = 'dummy'
        self.app_config.save()

    def test_articles_list(self):
        namespace = self.app_config.namespace
        response = self.client.get(
            reverse('{0}:article-list'.format(namespace)))
        self.assertContains(response, 'This is dummy article list page')

    def test_article_detail(self):
        article = self.create_article(app_config=self.app_config)
        namespace = self.app_config.namespace
        response = self.client.get(
            reverse(
                '{0}:article-detail'.format(namespace),
                kwargs={'slug': article.slug}
            ))
        self.assertContains(response, 'This is dummy article detail page')


class TestTranslationFallbacks(NewsBlogTestCase):
    def test_article_detail_not_translated_fallback(self):
        """
        If the fallback is configured, article is available in any
        (configured) language
        """
        author = self.create_person()
        code = "{0}-".format(self.language)

        with override(settings.LANGUAGES[0][0]):
            article = Article.objects.create(
                title=self.rand_str(),
                slug=self.rand_str(prefix=code),
                app_config=self.app_config,
                author=author, owner=author.user,
                publishing_date=now(),
                is_published=True,
            )
            article.save()
            article.categories.add(self.category1)
            url_one = reverse(
                'aldryn_newsblog:article-detail',
                kwargs={'slug': article.slug}
            )
        # Parler settings should be same as cms settings and vice versa
        # ensure that if hide_untranslated = True we don't have a fallback
        # redirect.
        LANGUAGES = add_default_language_settings(PARLER_LANGUAGES_HIDE)
        with override_parler_settings(PARLER_LANGUAGES=LANGUAGES):
            language = settings.LANGUAGES[1][0]
            with switch_language(article, language):
                slug = article.safe_translation_getter('slug', None,
                    language_code=language, any_language=True)
                url = reverse(
                    'aldryn_newsblog:article-detail',
                    kwargs={'slug': slug}
                )
                self.assertNotEquals(url, url_one)
                response = self.client.get(url)
                self.assertEquals(response.status_code, 404)

            # Test again with redirect_on_fallback = False
            with self.settings(CMS_LANGUAGES=self.NO_REDIRECT_CMS_SETTINGS):
                language = settings.LANGUAGES[1][0]
                with switch_language(article, language):
                    slug = article.safe_translation_getter('slug', None)
                    url = reverse(
                        'aldryn_newsblog:article-detail',
                        kwargs={'slug': slug, }
                    )
                    self.assertNotEquals(url, url_one)
                    response = self.client.get(url)
                    self.assertEquals(response.status_code, 404)

    def test_article_detail_not_translated_no_fallback(self):
        """
        If the fallback is disabled, article is available only in the
        language in which is translated
        """
        author = self.create_person()
        code = "{0}-".format(self.language)
        article = Article.objects.create(
            title=self.rand_str(), slug=self.rand_str(prefix=code),
            app_config=self.app_config,
            author=author, owner=author.user,
            publishing_date=now(),
            is_published=True,
        )
        article.save()
        article.categories.add(self.category1)

        PARLER_LANGUAGES = {
            1: (
                {'code': 'de'},
                {'code': 'fr'},
                {'code': 'en'},
            ),
            'default': {
                'hide_untranslated': True,
            }
        }
        LANGUAGES = add_default_language_settings(PARLER_LANGUAGES)
        with override_parler_settings(PARLER_LANGUAGES=LANGUAGES):

            # current language - it still exists
            article = Article.objects.get(pk=article.pk)
            language = settings.LANGUAGES[0][0]
            with switch_language(self.category1, language):
                url = reverse('aldryn_newsblog:article-detail',
                              kwargs={'slug': article.slug})
                response = self.client.get(url)
                self.assertContains(response, article.title)

            # non existing language - it does NOT exists
            language = settings.LANGUAGES[1][0]
            with switch_language(self.category1, language):
                url = reverse('aldryn_newsblog:article-detail',
                              kwargs={'slug': article.slug})
                response = self.client.get(url)
                self.assertEqual(response.status_code, 404)


class TestImages(NewsBlogTestCase):
    def test_article_detail_show_featured_image(self):
        author = self.create_person()
        with open(FEATURED_IMAGE_PATH, 'rb') as f:
            file_obj = DjangoFile(f, name='featured_image.jpg')
            image = Image.objects.create(owner=author.user,
                                         original_filename='featured_image.jpg',
                                         file=file_obj,
                                         subject_location='fooobar')
        article = self.create_article(author=author, featured_image=image)
        response = self.client.get(article.get_absolute_url())
        image_url = get_thumbnailer(article.featured_image).get_thumbnail({
            'size': (800, 450),
            'crop': True,
            'subject_location': article.featured_image.subject_location
        }).url
        self.assertContains(response, image_url)


class TestVariousViews(NewsBlogTestCase):
    def test_articles_by_tag(self):
        """
        Tests that TagArticleList view properly filters articles by their tags.

        This uses ANY of the languages articles are translated to.
        """

        untagged_articles = []
        for _ in range(5):
            article = self.create_article()
            untagged_articles.append(article)

        articles = self.create_tagged_articles(
            3, tags=(self.rand_str(), self.rand_str()))

        # tags are created in previous loop on demand, we need their slugs
        tag_slug1, tag_slug2 = articles.keys()
        url = reverse('aldryn_newsblog:article-list-by-tag',
                      kwargs={'tag': tag_slug2})
        response = self.client.get(url)
        for article in articles[tag_slug2]:
            self.assertContains(response, article.title)
        for article in articles[tag_slug1]:
            self.assertNotContains(response, article.title)
        for article in untagged_articles:
            self.assertNotContains(response, article.title)

    def test_articles_by_unknown_tag(self):
        response = self.client.get(reverse(
            'aldryn_newsblog:article-list-by-tag',
            kwargs={'tag': 'unknown'}))
        self.assertEqual(response.status_code, 404)

    def test_articles_count_by_month(self):
        months = [
            {'date': date(1914, 7, 3), 'num_articles': 1},
            {'date': date(1914, 8, 3), 'num_articles': 3},
            {'date': date(1945, 9, 3), 'num_articles': 5},
        ]
        for month in months:
            for _ in range(month['num_articles']):
                article = self.create_article(publishing_date=month['date'])

        # unpublish one specific article to test that it is not counted
        article.is_published = False
        article.save()
        months[-1]['num_articles'] -= 1

        self.assertEquals(
            sorted(
                Article.objects.get_months(
                    request=None, namespace=self.app_config.namespace
                ), key=itemgetter('num_articles')), months)

    def test_articles_count_by_author(self):
        authors = []
        for num_articles in [1, 3, 5]:
            person = self.create_person()
            person.num_articles = num_articles
            authors.append((person, num_articles))

        for i, data in enumerate(authors):
            for _ in range(data[1]):
                article = self.create_article(author=data[0])
            # replace author with it's pk, as we need it to easily compare
            authors[i] = (data[0].pk, data[1])

        # unpublish one specific article to test that it is not counted
        article.is_published = False
        article.save()
        authors[-1] = (authors[-1][0], authors[-1][1] - 1)

        self.assertEquals(
            sorted(
                Article.objects.get_authors(
                    namespace=self.app_config.namespace).values_list(
                        'pk', 'num_articles'),
                key=itemgetter(1)),
            authors)

    def test_articles_count_by_tags(self):
        tags = Article.objects.get_tags(
            request=None, namespace=self.app_config.namespace)
        self.assertEquals(tags, [])

        untagged_articles = []
        for _ in range(5):
            article = self.create_article()
            untagged_articles.append(article)

        # Tag objects are created on attaching tag name to Article,
        # so this looks not very DRY
        tag_names = ('tag foo', 'tag bar', 'tag buzz')
        # create unpublished article to test that it is not counted
        self.create_tagged_articles(
            1, tags=(tag_names[0],), is_published=False)
        tag_slug2 = list(self.create_tagged_articles(
            3, tags=(tag_names[1],)).keys())[0]
        tag_slug3 = list(self.create_tagged_articles(
            5, tags=(tag_names[2],)).keys())[0]
        tags_expected = [
            (tag_slug3, 5),
            (tag_slug2, 3),
        ]
        tags = Article.objects.get_tags(
            request=None, namespace=self.app_config.namespace)
        tags = [(tag.slug, tag.num_articles) for tag in tags]
        self.assertEquals(tags, tags_expected)

    def test_articles_by_date(self):
        in_articles = [
            self.create_article(
                publishing_date=datetime(
                    1914, 7, 28, randint(0, 23), randint(0, 59)))
            for _ in range(11)]
        out_articles = [
            self.create_article(
                publishing_date=datetime(
                    1939, 9, 1, randint(0, 23), randint(0, 59)))
            for _ in range(11)]
        response = self.client.get(reverse(
            'aldryn_newsblog:article-list-by-day',
            kwargs={'year': '1914', 'month': '07', 'day': '28'}))
        for article in out_articles:
            self.assertNotContains(response, article.title)
        for article in in_articles:
            self.assertContains(response, article.title)

    def test_articles_by_month(self):
        in_articles = [
            self.create_article(
                publishing_date=datetime(
                    1914, 7, randint(1, 31), randint(0, 23), randint(0, 59)))
            for _ in range(11)]
        out_articles = [
            self.create_article(
                publishing_date=datetime(
                    1939, 9, 1, randint(0, 23), randint(0, 59)))
            for _ in range(11)]
        response = self.client.get(reverse(
            'aldryn_newsblog:article-list-by-month',
            kwargs={'year': '1914', 'month': '07'}))
        for article in out_articles:
            self.assertNotContains(response, article.title)
        for article in in_articles:
            self.assertContains(response, article.title)

    def test_articles_by_year(self):
        in_articles = [
            self.create_article(
                publishing_date=datetime(
                    1914, randint(1, 11), randint(1, 28),
                    randint(0, 23), randint(0, 59)))
            for _ in range(11)]
        out_articles = [
            self.create_article(
                publishing_date=datetime(
                    1939, randint(1, 12), randint(1, 28),
                    randint(0, 23), randint(0, 59)))
            for _ in range(11)]
        response = self.client.get(reverse(
            'aldryn_newsblog:article-list-by-year', kwargs={'year': '1914'}))
        for article in out_articles:
            self.assertNotContains(response, article.title)
        for article in in_articles:
            self.assertContains(response, article.title)

    def test_unattached_namespace(self):
        # create a new namespace that has no corresponding blog app page
        app_config = NewsBlogConfig.objects.create(namespace='another')
        articles = [self.create_article(app_config=app_config)
                    for _ in range(11)]
        with self.assertRaises(NoReverseMatch):
            self.client.get(articles[0].get_absolute_url())


class TestIndex(NewsBlogTestCase):
    def test_index_simple(self):
        self.request = self.get_request('en')
        self.index = ArticleIndex()
        content0 = self.rand_str(prefix='content0_')
        self.setup_categories()

        article = self.create_article(content=content0, lead_in='lead in text',
                                      title='a title')
        article.categories.add()
        for tag_name in ('tag 1', 'tag2'):
            article.tags.add(tag_name)
        for category in (self.category1, self.category2):
            article.categories.add(category)
        article.save()

        self.assertEqual(self.index.get_title(article), 'a title')
        self.assertEqual(self.index.get_description(article), 'lead in text')
        self.assertTrue('lead in text' in self.index.get_search_data(
            article, 'en', self.request))
        self.assertTrue(content0 in self.index.get_search_data(
            article, 'en', self.request))
        self.assertTrue('tag 1' in self.index.get_search_data(
            article, 'en', self.request))
        self.assertTrue(self.category1.name in self.index.get_search_data(
            article, 'en', self.request))

    def test_index_multilingual(self):
        self.index = ArticleIndex()
        content0 = self.rand_str(prefix='content0_')
        self.setup_categories()

        article_1 = self.create_article(
            content=content0, lead_in=u'lead in text', title=u'a title')
        article_2 = self.create_article(
            content=content0, lead_in=u'lead in text', title=u'second title')
        for article in (article_1, article_2):
            for tag_name in ('tag 1', 'tag2'):
                article.tags.add(tag_name)
            for category in (self.category1, self.category2):
                article.categories.add(category)
        with switch_language(article_2, 'de'):
            article_2.title = u'de title'
            article_2.lead_in = u'de lead in'
            article_2.save()

        LANGUAGES = add_default_language_settings(PARLER_LANGUAGES_HIDE)
        with override_parler_settings(PARLER_LANGUAGES=LANGUAGES):
            with smart_override('de'):
                language = get_current_language()
                # english-only article is excluded
                qs = self.index.index_queryset(language)
                self.assertEqual(qs.count(), 1)
                self.assertEqual(
                    qs.translated(language, title__icontains='title').count(),
                    1
                )
                # the language is correctly setup
                for article_de in qs:
                    self.assertEqual(
                        self.index.get_title(article_de), 'de title')
                    self.assertEqual(
                        self.index.get_description(article_de), 'de lead in')


class ViewLanguageFallbackMixin(object):
    view_name = None
    view_kwargs = {}

    def get_view_kwargs(self):
        """
        Prepare and return kwargs to resolve view
        :return: dict
        """
        return {}.update(self.view_kwargs)

    def create_authors(self):
        self.author = self.create_person()
        self.owner = self.author.user
        return self.author, self.owner

    def create_de_article(self, author=None, owner=None, app_config=None,
                          categories=None):
        if author is None:
            author = self.author
        if owner is None:
            owner = self.owner
        if app_config is None:
            app_config = self.app_config

        with force_language('de'):
            de_article = Article.objects.create(
                title='a DE title',
                slug='a-de-title',
                lead_in='DE lead in text',
                author=author,
                owner=owner,
                app_config=app_config,
                publishing_date=now(),
                is_published=True,
            )
        if categories:
            de_article.categories = categories
        de_article.tags.add('tag1')
        de_article.save()
        return de_article

    def create_en_articles(self, author=None, owner=None, app_config=None,
                           amount=3, categories=None):
        if author is None:
            author = self.author
        if owner is None:
            owner = self.owner
        if app_config is None:
            app_config = self.app_config

        with force_language('en'):
            articles = []
            for _ in range(amount):
                article = self.create_article(author=author,
                                              owner=owner,
                                              app_config=app_config)
                if categories:
                    article.categories = categories
                article.tags.add('tag1')
                article.save()
                articles.append(article)
        return articles

    def test_a0_en_only(self):
        namespace = self.app_config.namespace
        self.page.unpublish('de')
        author, owner = self.create_authors()
        author.translations.create(
            slug='{0}-de'.format(author.slug),
            language_code='de')
        de_article = self.create_de_article(
            author=author,
            owner=owner,
            categories=[self.category1],
        )
        articles = self.create_en_articles(categories=[self.category1])
        with force_language('en'):
            response = self.client.get(
                reverse(
                    '{0}:{1}'.format(namespace, self.view_name),
                    kwargs=self.get_view_kwargs()
                )
            )
        for article in articles:
            self.assertContains(response, article.title)
        self.assertNotContains(response, de_article.title)

    def test_a1_en_de(self):
        namespace = self.app_config.namespace
        author, owner = self.create_authors()
        author.translations.create(
            slug='{0}-de'.format(author.slug),
            language_code='de')
        de_article = self.create_de_article(
            author=author,
            owner=owner,
            categories=[self.category1]
        )
        articles = self.create_en_articles(categories=[self.category1])
        with force_language('en'):
            response = self.client.get(
                reverse(
                    '{0}:{1}'.format(namespace, self.view_name),
                    kwargs=self.get_view_kwargs()
                )
            )
        for article in articles:
            self.assertContains(response, article.title)
        self.assertContains(response, de_article.title)


class ArticleListViewLanguageFallback(ViewLanguageFallbackMixin,
                                      NewsBlogTestCase):
    view_name = 'article-list'


class LatestArticlesFeedLanguageFallback(ViewLanguageFallbackMixin,
                                         NewsBlogTestCase):
    view_name = 'article-list-feed'


class YearArticleListLanguageFallback(ViewLanguageFallbackMixin,
                                      NewsBlogTestCase):
    view_name = 'article-list-by-year'

    def get_view_kwargs(self):
        return {'year': now().year}


class MonthArticleListLanguageFallback(ViewLanguageFallbackMixin,
                                       NewsBlogTestCase):
    view_name = 'article-list-by-month'

    def get_view_kwargs(self):
        kwargs = {
            'year': now().year,
            'month': now().month,
        }
        return kwargs


class DayArticleListLanguageFallback(ViewLanguageFallbackMixin,
                                     NewsBlogTestCase):
    view_name = 'article-list-by-day'

    def get_view_kwargs(self):
        kwargs = {
            'year': now().year,
            'month': now().month,
            'day': now().day,
        }
        return kwargs


class AuthorArticleListLanguageFallback(ViewLanguageFallbackMixin,
                                        NewsBlogTestCase):
    view_name = 'article-list-by-author'

    def get_view_kwargs(self):
        kwargs = {
            'author': self.author.slug
        }
        return kwargs


class CategoryArticleListLanguageFallback(ViewLanguageFallbackMixin,
                                          NewsBlogTestCase):
    view_name = 'article-list-by-category'

    def get_view_kwargs(self):
        kwargs = {
            'category': self.category1.slug
        }
        return kwargs


class CategoryFeedListLanguageFallback(ViewLanguageFallbackMixin,
                                       NewsBlogTestCase):
    view_name = 'article-list-by-category-feed'

    def get_view_kwargs(self):
        kwargs = {
            'category': self.category1.slug
        }
        return kwargs


class TagArticleListLanguageFallback(ViewLanguageFallbackMixin,
                                     NewsBlogTestCase):
    view_name = 'article-list-by-tag'

    def get_view_kwargs(self):
        kwargs = {
            'tag': 'tag1'
        }
        return kwargs


class TagFeedLanguageFallback(ViewLanguageFallbackMixin,
                              NewsBlogTestCase):
    view_name = 'article-list-by-tag-feed'

    def get_view_kwargs(self):
        kwargs = {
            'tag': 'tag1'
        }
        return kwargs



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from django.core.urlresolvers import reverse

from reversion.revisions import default_revision_manager
try:
    from reversion import create_revision
except ImportError:
    # django-reversion >=1.9
    from reversion.revisions import create_revision

import six

from django.db import transaction

from aldryn_reversion.core import create_revision as aldryn_create_revision

from parler.utils.context import switch_language

from . import NewsBlogTestCase
from aldryn_newsblog.cms_appconfig import NewsBlogConfig


class TestVersioning(NewsBlogTestCase):
    def create_revision(self, article, content=None, language=None, **kwargs):
        with transaction.atomic():
            with create_revision():
                for k, v in six.iteritems(kwargs):
                    setattr(article, k, v)
                if content:
                    plugins = article.content.get_plugins()
                    plugin = plugins[0].get_plugin_instance()[0]
                    plugin.body = content
                    plugin.save()
                # TODO: Cover both cases (plugin modification/recreation)
                # if content:
                #     article.content.get_plugins().delete()
                #     api.add_plugin(article.content, 'TextPlugin',
                #                    self.language, body=content)
                article.save()

    def revert_to(self, article, revision):
        (default_revision_manager.get_for_object(article)[revision]
                                 .revision.revert())

    def test_revert_revision(self):
        title1 = self.rand_str(prefix='title1_')
        title2 = self.rand_str(prefix='title2_')

        content0 = self.rand_str(prefix='content0_')
        content1 = self.rand_str(prefix='content1_')
        content2 = self.rand_str(prefix='content2_')

        article = self.create_article(content=content0)

        # Revision 1
        self.create_revision(article, title=title1, content=content1)

        response = self.client.get(article.get_absolute_url())
        self.assertContains(response, title1)
        self.assertContains(response, content1)
        self.assertNotContains(response, content0)

        # Revision 2
        self.create_revision(article, title=title2, content=content2)

        response = self.client.get(article.get_absolute_url())
        self.assertContains(response, title2)
        self.assertContains(response, content2)
        self.assertNotContains(response, content1)

        # Revert to revision 1
        self.revert_to(article, 1)

        response = self.client.get(article.get_absolute_url())
        self.assertContains(response, title1)
        self.assertContains(response, content1)
        self.assertNotContains(response, content0)
        self.assertNotContains(response, content2)

    def test_revert_translated_revision(self):
        title1_en = self.rand_str(prefix='title1_en_')
        title1_de = self.rand_str(prefix='title1_de_')
        title2_en = self.rand_str(prefix='title2_en_')
        title2_de = self.rand_str(prefix='title2_de_')

        article = self.create_article()

        # Revision 1
        article.set_current_language('en')
        self.create_revision(article, title=title1_en)

        article.set_current_language('de')
        self.create_revision(article, title=title1_de)

        with switch_language(article, 'en'):
            response = self.client.get(article.get_absolute_url())
            self.assertContains(response, title1_en)

        with switch_language(article, 'de'):
            response = self.client.get(article.get_absolute_url())
            self.assertContains(response, title1_de)

        # Revision 2a (modify just EN)
        article.set_current_language('en')
        self.create_revision(article, title=title2_en)

        response = self.client.get(article.get_absolute_url())
        self.assertContains(response, title2_en)

        with switch_language(article, 'de'):
            response = self.client.get(article.get_absolute_url())
            self.assertContains(response, title1_de)

        # Revision 2b (modify just DE)
        article.set_current_language('de')
        self.create_revision(article, title=title2_de)

        with switch_language(article, 'en'):
            response = self.client.get(article.get_absolute_url())
            self.assertContains(response, title2_en)

        with switch_language(article, 'de'):
            response = self.client.get(article.get_absolute_url())
            self.assertContains(response, title2_de)

        # Revert to revision 2a (EN=2, DE=1)
        self.revert_to(article, 1)

        with switch_language(article, 'en'):
            response = self.client.get(article.get_absolute_url())
            self.assertContains(response, title2_en)

        with switch_language(article, 'de'):
            response = self.client.get(article.get_absolute_url())
            self.assertContains(response, title1_de)

        # Revert to revision 1 (EN=1, DE=1)
        self.revert_to(article, 2)

        with switch_language(article, 'en'):
            response = self.client.get(article.get_absolute_url())
            self.assertContains(response, title1_en)

        with switch_language(article, 'de'):
            response = self.client.get(article.get_absolute_url())
            self.assertContains(response, title1_de)

    def test_edit_plugin_directly(self):
        content0 = self.rand_str(prefix='content0_')
        content1 = self.rand_str(prefix='content1_')
        content2 = self.rand_str(prefix='content2_')

        article = self.create_article(content=content0)

        # Revision 1
        self.create_revision(article, content=content1)

        self.assertEqual(
            len(default_revision_manager.get_for_object(article)), 1)

        # Revision 2
        with transaction.atomic():
            plugins = article.content.get_plugins()
            plugin = plugins[0].get_plugin_instance()[0]
            plugin.body = content2
            plugin.save()
            aldryn_create_revision(article)

        self.assertEqual(
            len(default_revision_manager.get_for_object(article)), 2)

        response = self.client.get(article.get_absolute_url())
        self.assertContains(response, content2)
        self.assertNotContains(response, content1)

        # Revert to revision 1
        self.revert_to(article, 1)

        response = self.client.get(article.get_absolute_url())
        self.assertContains(response, content1)
        self.assertNotContains(response, content2)

    def test_blog_config_recovery_accessible(self):
        with transaction.atomic():
            with create_revision():
                new_conf = NewsBlogConfig(
                    namespace='test_revocery_admin_url', paginate_by=15)
                new_conf.save()
        new_config_version = (default_revision_manager
                              .get_for_object(new_conf)[0])
        new_config_pk = new_conf.pk
        self.assertEqual(NewsBlogConfig.objects.filter(
            pk=new_config_pk).count(), 1)
        new_conf.delete()
        self.assertEqual(NewsBlogConfig.objects.filter(
            pk=new_config_pk).count(), 0)

        # check that there is a a way to access recovery view
        obj = new_config_version.object_version.object
        opts = obj._meta
        url = reverse(
            'admin:{0}_{1}_{2}'.format(
                opts.app_label,
                obj._meta.model_name,
                'recover'),
            args=[new_config_version.pk])
        # ust in case check the length, but at this step either a
        # NoReverseMatch should occur or other error,
        # if no exception is raised, it is a good sign
        self.assertGreater(len(url), 4)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import NoReverseMatch
from django.utils.translation import override

from . import NewsBlogTestCase


class TestI18N(NewsBlogTestCase):

    def test_absolute_url_fallback(self):
        # Create an EN article
        with override('en'):
            article = self.create_article(
                title='God Save the Queen!', slug='god-save-queen')
        # Add a DE translation
        article.create_translation('de',
            title='Einigkeit und Recht und Freiheit!',
            slug='einigkeit-und-recht-und-freiheit')

        # Reload for good measure
        article = self.reload(article)

        self.assertEquals(article.get_absolute_url(language='en'),
            '/en/page/god-save-queen/')
        # Test that we can request the other defined language too
        self.assertEquals(article.get_absolute_url(language='de'),
            '/de/page/einigkeit-und-recht-und-freiheit/')

        # Now, let's request a language that article has not yet been translated
        # to, but has fallbacks defined, we should get EN
        self.assertEquals(article.get_absolute_url(language='fr'),
            '/en/page/god-save-queen/')

        # With settings changed to 'redirect_on_fallback': False, test again.
        with self.settings(CMS_LANGUAGES=self.NO_REDIRECT_CMS_SETTINGS):
            self.assertEquals(article.get_absolute_url(language='fr'),
                '/fr/page/god-save-queen/')

        # Now, let's request a language that has a fallback defined, but it is
        # not available either (should raise NoReverseMatch)
        with self.assertRaises(NoReverseMatch):
            article.get_absolute_url(language='it')



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from . import NewsBlogTestCase
from aldryn_newsblog.sitemaps import NewsBlogSitemap

try:
    from django.contrib.sites.shortcuts import get_current_site
except ImportError:
    # Django 1.6
    from django.contrib.sites.models import get_current_site
from django.utils.translation import override


class TestSitemaps(NewsBlogTestCase):

    def _sitemap_urls(self, sitemap):
        urls_info = sitemap.get_urls()
        urls = [url_info['location'] for url_info in urls_info]
        return urls

    def _article_urls(self, articles, lang):
        self.request = self.get_request(lang)
        host = 'http://' + get_current_site(self.request).domain
        return [host + article.get_absolute_url(lang) for article in articles]

    def assertArticlesIn(self, articles, sitemap):
        urls = self._sitemap_urls(sitemap)
        article_urls = self._article_urls(articles, sitemap.language)

        for url in article_urls:
            self.assertIn(url, urls)

    def assertArticlesNotIn(self, articles, sitemap):
        urls = self._sitemap_urls(sitemap)
        article_urls = self._article_urls(articles, sitemap.language)

        for url in article_urls:
            self.assertNotIn(url, urls)

    def assertSitemapLanguage(self, sitemap, lang):
        self.request = self.get_request(lang)
        urls = self._sitemap_urls(sitemap)
        host = 'http://' + get_current_site(self.request).domain
        url_start = "{0}/{1}".format(host, lang)

        for url in urls:
            self.assertTrue(url.startswith(url_start))

    def test_listening_all_instances(self):
        articles = [self.create_article() for _ in range(11)]
        unpublished_article = articles[0]
        unpublished_article.is_published = False
        unpublished_article.save()
        sitemap = NewsBlogSitemap()
        self.assertArticlesIn(articles[1:], sitemap)
        self.assertArticlesNotIn([unpublished_article], sitemap)

    def test_listening_namespace(self):
        articles = [self.create_article() for _ in range(11)]
        unpublished_article = articles[0]
        unpublished_article.is_published = False
        unpublished_article.save()
        sitemap = NewsBlogSitemap(namespace=self.app_config.namespace)
        self.assertArticlesIn(articles[1:], sitemap)
        self.assertArticlesNotIn([unpublished_article], sitemap)

    def test_listening_unexisting_namespace(self):
        articles = [self.create_article() for _ in range(11)]
        unpublished_article = articles[0]
        unpublished_article.is_published = False
        unpublished_article.save()
        sitemap = NewsBlogSitemap(
            namespace='not exists')
        self.assertFalse(sitemap.items())
        self.assertArticlesNotIn(articles, sitemap)

    def test_languages_support(self):
        with override('en'):
            multilanguage_article = self.create_article()
            en_article = self.create_article()

        multilanguage_article.create_translation(
            'de', title='DE title', slug='de-article')
        with override('de'):
            de_article = self.create_article()

        en_sitemap = NewsBlogSitemap(language='en')
        self.assertArticlesIn([multilanguage_article, en_article], en_sitemap)
        self.assertArticlesNotIn([de_article], en_sitemap)
        self.assertSitemapLanguage(en_sitemap, 'en')

        de_sitemap = NewsBlogSitemap(language='de')
        self.assertArticlesIn([multilanguage_article, de_article], de_sitemap)
        self.assertArticlesNotIn([en_article], de_sitemap)
        self.assertSitemapLanguage(de_sitemap, 'de')



# -*- coding: utf-8 -*-
from django.utils.translation import activate

from aldryn_newsblog.search_indexes import ArticleIndex

from . import NewsBlogTestCase


class ArticleIndexingTests(NewsBlogTestCase):

    def get_index(self):
        from haystack.constants import DEFAULT_ALIAS

        index = ArticleIndex()
        index._backend_alias = DEFAULT_ALIAS
        return index

    def test_article_is_indexed_using_prepare(self):
        activate(self.language)

        lead_in = 'Hello! this text will be searchable.'

        article = self.create_article(lead_in=lead_in)

        index = self.get_index()

        data = index.prepare(article)

        count = data['text'].count(lead_in)

        self.assertTrue(count != 0, "Couldn't find %s in text" % lead_in)

    def test_translated_article_is_indexed_using_prepare(self):
        activate(self.language)

        lead_in = 'Hello! this text will be searchable.'

        # create english article
        article = self.create_article(lead_in=lead_in)

        # create german translation for article
        article.set_current_language('de')
        article.title = '%s [de]' % self.rand_str()
        article.save()

        index = self.get_index()

        data = index.prepare(article)

        self.assertEquals(data['language'], 'de')
        self.assertEquals(data['url'], article.get_absolute_url('de'))

    def test_article_not_indexed_if_no_translation(self):
        index = self.get_index()
        # create english article
        article = self.create_article()

        # should the index be updated for this object? (yes)
        should_update = index.should_update(article)
        self.assertEquals(should_update, True)

        # remove all translations for article
        article.translations.all().delete()

        # should the index be updated for this object? (no)
        should_update = index.should_update(article)
        self.assertEquals(should_update, False)



import django.forms
from . import models
from django.utils.translation import ugettext_lazy as _


class LightsliderPluginForm(django.forms.ModelForm):

    class Meta:
        fields = [
            'number_of_concurrent_slides',
            'number_of_paginating_slides',
            'style',
            'transition_style',
            'transition_effect',
            'ride',
            'interval',
        ]
        model = models.LightsliderPlugin

    def clean_style(self):
        style = self.cleaned_data.get('style')
        template = 'aldryn_lightslider/plugins/lightslider/{}/lightslider.html'.format(
            style)
        # Check if template for style exists:
        try:
            django.template.loader.select_template([template])
        except django.template.TemplateDoesNotExist:
            raise django.forms.ValidationError(
                _("Not a valid style (Template %s does not exist)") % template
            )
        return style


class LightsliderSlidePluginForm(django.forms.ModelForm):

    class Meta:
        fields = ['image', 'title', 'classes', ]
        model = models.LightsliderSlidePlugin




# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.test import TransactionTestCase

from aldryn_newsblog.cms_appconfig import NewsBlogConfig
from aldryn_newsblog.models import Article
from aldryn_people.models import Person

from . import NewsBlogTestsMixin


class AdminTest(NewsBlogTestsMixin, TransactionTestCase):

    def test_admin_owner_default(self):
        from django.contrib import admin
        admin.autodiscover()
        # since we now have data migration to create the default
        # NewsBlogConfig (if migrations were not faked, django >1.7)
        # we need to delete one of configs to be sure that it is pre selected
        # in the admin view.
        if NewsBlogConfig.objects.count() > 1:
            # delete the app config that was created during test set up.
            NewsBlogConfig.objects.filter(namespace='NBNS').delete()
        user = self.create_user()
        user.is_superuser = True
        user.save()

        Person.objects.create(user=user, name=u' '.join(
            (user.first_name, user.last_name)))

        admin_inst = admin.site._registry[Article]
        self.request = self.get_request('en')
        self.request.user = user
        self.request.META['HTTP_HOST'] = 'example.com'
        response = admin_inst.add_view(self.request)
        option = '<option value="1" selected="selected">%s</option>'
        self.assertContains(response, option % user.username)
        self.assertContains(response, option % user.get_full_name())



HELPER_SETTINGS={
    'INSTALLED_APPS': [
        'djangocms_admin_style',
        'mptt',
		'cms',
		'menus',
		'sekizai',
		'cmsplugin_filer_file',
	    'cmsplugin_filer_folder',
	    'cmsplugin_filer_image',
	    'cmsplugin_filer_utils',
    ],
}


# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import os
import random
import string
import sys

from django.conf import settings
from django.contrib.auth.models import AnonymousUser, User
from django.core.urlresolvers import clear_url_caches
from django.core.cache import cache
from django.test import RequestFactory
from django.utils.timezone import now
from django.utils.translation import override
from aldryn_categories.models import Category
from aldryn_newsblog.cms_apps import NewsBlogApp
from aldryn_newsblog.models import Article, NewsBlogConfig
from aldryn_people.models import Person
from cms import api
from cms.apphook_pool import apphook_pool
from cms.appresolver import clear_app_resolvers
from cms.exceptions import AppAlreadyRegistered
from cms.test_utils.testcases import CMSTestCase, TransactionCMSTestCase
from cms.utils import get_cms_setting

from cms.toolbar.toolbar import CMSToolbar


from parler.utils.context import switch_language

TESTS_ROOT = os.path.abspath(os.path.dirname(__file__))
TESTS_STATIC_ROOT = os.path.abspath(os.path.join(TESTS_ROOT, 'static'))


class NewsBlogTestsMixin(object):

    NO_REDIRECT_CMS_SETTINGS = {
        1: [
            {
                'code': 'de',
                'name': 'Deutsche',
                'fallbacks': ['en', ]  # FOR TESTING DO NOT ADD 'fr' HERE
            },
            {
                'code': 'fr',
                'name': 'Française',
                'fallbacks': ['en', ]  # FOR TESTING DO NOT ADD 'de' HERE
            },
            {
                'code': 'en',
                'name': 'English',
                'fallbacks': ['de', 'fr', ]
            },
            {
                'code': 'it',
                'name': 'Italiano',
                'fallbacks': ['fr', ]  # FOR TESTING, LEAVE AS ONLY 'fr'
            },
        ],
        'default': {
            'redirect_on_fallback': False,
        }
    }

    @staticmethod
    def reload(node):
        """NOTE: django-treebeard requires nodes to be reloaded via the Django
        ORM once its sub-tree is modified for the API to work properly.

        See:: https://tabo.pe/projects/django-treebeard/docs/2.0/caveats.html

        This is a simple helper-method to do that."""
        return node.__class__.objects.get(id=node.id)

    @classmethod
    def rand_str(cls, prefix=u'', length=23, chars=string.ascii_letters):
        return prefix + u''.join(random.choice(chars) for _ in range(length))

    @classmethod
    def create_user(cls):
        return User.objects.create(
            username=cls.rand_str(), first_name=cls.rand_str(),
            last_name=cls.rand_str())

    def create_person(self):
        return Person.objects.create(
            user=self.create_user(), slug=self.rand_str())

    def create_article(self, content=None, **kwargs):
        try:
            author = kwargs['author']
        except KeyError:
            author = self.create_person()
        try:
            owner = kwargs['owner']
        except KeyError:
            owner = author.user

        fields = {
            'title': self.rand_str(),
            'slug': self.rand_str(),
            'author': author,
            'owner': owner,
            'app_config': self.app_config,
            'publishing_date': now(),
            'is_published': True,
        }

        fields.update(kwargs)

        article = Article.objects.create(**fields)
        # save again to calculate article search_data.
        article.save()

        if content:
            api.add_plugin(article.content, 'TextPlugin',
                           self.language, body=content)
        return article

    def create_tagged_articles(self, num_articles=3, tags=('tag1', 'tag2'),
                               **kwargs):
        """Create num_articles Articles for each tag"""
        articles = {}
        for tag_name in tags:
            tagged_articles = []
            for _ in range(num_articles):
                article = self.create_article(**kwargs)
                article.save()
                article.tags.add(tag_name)
                tagged_articles.append(article)
            tag_slug = tagged_articles[0].tags.slugs()[0]
            articles[tag_slug] = tagged_articles
        return articles

    def setup_categories(self):
        """
        Sets-up i18n categories (self.category_root, self.category1 and
        self.category2) for use in tests
        """
        self.language = settings.LANGUAGES[0][0]

        categories = []
        # Set the default language, create the objects
        with override(self.language):
            code = "{0}-".format(self.language)
            self.category_root = Category.add_root(
                name=self.rand_str(prefix=code, length=8))
            categories.append(self.category_root)
            self.category1 = self.category_root.add_child(
                name=self.rand_str(prefix=code, length=8))
            categories.append(self.category1)
            self.category2 = self.category_root.add_child(
                name=self.rand_str(prefix=code, length=8))
            categories.append(self.category2)

        # We should reload category_root, since we modified its children.
        self.category_root = self.reload(self.category_root)

        # Setup the other language(s) translations for the categories
        for language, _ in settings.LANGUAGES[1:]:
            for category in categories:
                with switch_language(category, language):
                    code = "{0}-".format(language)
                    category.name = self.rand_str(prefix=code, length=8)
                    category.save()

    @staticmethod
    def get_request(language=None, url="/"):
        """
        Returns a Request instance populated with cms specific attributes.
        """
        request_factory = RequestFactory(HTTP_HOST=settings.ALLOWED_HOSTS[0])
        request = request_factory.get(url)
        request.session = {}
        request.LANGUAGE_CODE = language or settings.LANGUAGE_CODE
        # Needed for plugin rendering.
        request.current_page = None
        request.user = AnonymousUser()
        request.toolbar = CMSToolbar(request)
        return request

    def setUp(self):
        self.template = get_cms_setting('TEMPLATES')[0][0]
        self.language = settings.LANGUAGES[0][0]
        self.root_page = api.create_page(
            'root page', self.template, self.language, published=True)
        self.app_config = NewsBlogConfig.objects.create(
            namespace='NBNS', paginate_by=15)
        self.page = api.create_page(
            'page', self.template, self.language, published=True,
            parent=self.root_page,
            apphook='NewsBlogApp',
            apphook_namespace=self.app_config.namespace)
        self.plugin_page = api.create_page(
            title="plugin_page", template=self.template, language=self.language,
            parent=self.root_page, published=True)
        self.placeholder = self.page.placeholders.all()[0]

        self.setup_categories()

        for page in self.root_page, self.page:
            for language, _ in settings.LANGUAGES[1:]:
                api.create_title(language, page.get_slug(), page)
                page.publish(language)


class CleanUpMixin(object):
    apphook_object = None

    def tearDown(self):
        """
        Do a proper cleanup, delete everything what is preventing us from
        clean environment for tests.
        :return: None
        """
        self.app_config.delete()
        self.reset_all()
        cache.clear()
        super(CleanUpMixin, self).tearDown()

    def get_apphook_object(self):
        return self.apphook_object

    def reset_apphook_cmsapp(self, apphook_object=None):
        """
        For tests that should not be polluted by previous setup we need to
        ensure that app hooks are reloaded properly. One of the steps is to
        reset the relation between EventListAppHook and EventsConfig
        """
        if apphook_object is None:
            apphook_object = self.get_apphook_object()
        app_config = getattr(apphook_object, 'app_config', None)
        if (app_config and
                getattr(app_config, 'cmsapp', None)):
            delattr(apphook_object.app_config, 'cmsapp')
        if getattr(app_config, 'cmsapp', None):
            delattr(app_config, 'cmsapp')

    def reset_all(self):
        """
        Reset all that could leak from previous test to current/next test.
        :return: None
        """
        apphook_object = self.get_apphook_object()
        self.delete_app_module(apphook_object.__module__)
        self.reload_urls(apphook_object)
        self.apphook_clear()

    def delete_app_module(self, app_module=None):
        """
        Remove APP_MODULE from sys.modules. Taken from cms.
        :return: None
        """
        if app_module is None:
            apphook_object = self.get_apphook_object()
            app_module = apphook_object.__module__
        if app_module in sys.modules:
            del sys.modules[app_module]

    def apphook_clear(self):
        """
        Clean up apphook_pool and sys.modules. Taken from cms with slight
        adjustments and fixes.
        :return: None
        """
        try:
            apphooks = apphook_pool.get_apphooks()
        except AppAlreadyRegistered:
            # there is an issue with discover apps, or i'm using it wrong.
            # setting discovered to True solves it. Maybe that is due to import
            # from aldryn_events.cms_apps which registers EventListAppHook
            apphook_pool.discovered = True
            apphooks = apphook_pool.get_apphooks()

        for name, label in list(apphooks):
            if apphook_pool.apps[name].__class__.__module__ in sys.modules:
                del sys.modules[apphook_pool.apps[name].__class__.__module__]
        apphook_pool.clear()
        self.reset_apphook_cmsapp()

    def reload_urls(self, apphook_object=None):
        """
        Clean up url related things (caches, app resolvers, modules).
        Taken from cms.
        :return: None
        """
        if apphook_object is None:
            apphook_object = self.get_apphook_object()
        app_module = apphook_object.__module__
        package = app_module.split('.')[0]
        clear_app_resolvers()
        clear_url_caches()
        url_modules = [
            'cms.urls',
            '{0}.urls'.format(package),
            settings.ROOT_URLCONF
        ]

        for module in url_modules:
            if module in sys.modules:
                del sys.modules[module]


class NewsBlogTestCase(CleanUpMixin, NewsBlogTestsMixin, CMSTestCase):
    apphook_object = NewsBlogApp
    pass


class NewsBlogTransactionTestCase(CleanUpMixin,
                                  NewsBlogTestsMixin,
                                  TransactionCMSTestCase):
    apphook_object = NewsBlogApp
    pass



# -*- coding: utf-8 -*-
from functools import partial

import filer
from cms.models.pluginmodel import CMSPlugin
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.html import strip_tags
from django.utils.translation import ugettext_lazy as _

import collections
import os
import django.forms.models

from . import utils, constants, fields

##########
# Mixins #  do NOT use outside of this package!
##########  Because changes here might require Database migrations!

CMSPluginField = partial(
    models.OneToOneField,
    to=CMSPlugin,
    related_name='+',
    parent_link=True,
)


class Classes(models.TextField):
    # TODO: validate
    default_field_class = fields.Classes

    def __init__(self, *args, **kwargs):
        if 'blank' not in kwargs:
            kwargs['blank'] = True
        if 'default' not in kwargs:
            kwargs['default'] = ''
        if 'help_text' not in kwargs:
            kwargs['help_text'] = 'space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>'
        super(Classes, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': self.default_field_class,
        }
        defaults.update(kwargs)
        return super(Classes, self).formfield(**defaults)


# Create your models here.
@python_2_unicode_compatible
class LightsliderPlugin(CMSPlugin):
    STYLE_DEFAULT = 'standard'

    TRANSITION_SLIDE = 'slide'
    TRANSITION_FADE = 'fade'

    TRANSITION_EFFECT_LINEAR = 'linear'
    TRANSITION_EFFECT_EASE = 'ease'
    TRANSITION_EFFECT_EASE_IN = 'ease-in'
    TRANSITION_EFFECT_EASE_OUT = 'ease-out'
    TRANSITION_EFFECT_EASE_IN_OUT = 'ease-in-out'

    STYLE_CHOICES = [
        (STYLE_DEFAULT, _('Standard')),
    ]

    TRANSITION_STYLE_CHOICES = (
        (TRANSITION_SLIDE, _('Slide')),
        (TRANSITION_FADE, _('Fade')),
    )

    TRANSITION_EFFECT_CHOICES = (
        (TRANSITION_EFFECT_LINEAR, _('Linear')),
        (TRANSITION_EFFECT_EASE, _('Ease')),
        (TRANSITION_EFFECT_EASE_IN, _('Ease in')),
        (TRANSITION_EFFECT_EASE_OUT, _('Ease out')),
        (TRANSITION_EFFECT_EASE_IN_OUT, _('Ease in out')),
    )

    cmsplugin_ptr = CMSPluginField()

    number_of_concurrent_slides = models.PositiveIntegerField(
        _('Number of concurrent slides'),
        default=3,
        help_text=_('How many images displayed '
                    'at the same time.'),
    )

    number_of_paginating_slides = models.PositiveIntegerField(
        _('Number of paginating slides'),
        default=1,
        help_text=_('How many images to slide out '
                    'when going forward or backward.'),
    )

    style = models.CharField(
        _('Style'),
        choices=STYLE_CHOICES + utils.get_additional_styles(),
        default=STYLE_DEFAULT,
        max_length=50,
    )
    aspect_ratio = models.CharField(
        _("aspect ratio"),
        max_length=10,
        blank=True,
        default='',
        choices=constants.ASPECT_RATIO_CHOICES
    )
    transition_style = models.CharField(
        _('Transition Style'),
        choices=TRANSITION_STYLE_CHOICES,
        default='',
        max_length=50,
        blank=True,
    )
    transition_effect = models.CharField(
        _('Transition Effect'),
        choices=TRANSITION_EFFECT_CHOICES,
        default='',
        max_length=50,
        blank=True,
    )
    ride = models.BooleanField(
        _('Ride'),
        default=True,
        help_text=_('Whether to mark the carousel as animating '
                    'starting at page load.'),
    )
    interval = models.IntegerField(
        _('Interval'),
        default=5000,
        help_text=_("The amount of time to delay between automatically "
                    "cycling an item."),
    )
    wrap = models.BooleanField(
        default=True,
        blank=True,
        help_text=_('Whether the carousel should cycle continuously or '
                    'have hard stops.')
    )
    pause = models.BooleanField(
        default=True,
        blank=True,
        help_text=_('Pauses the cycling of the carousel on mouseenter and '
                    'resumes the cycling of the carousel on mouseleave.')
    )
    classes = Classes()

    def __str__(self):
        data = django.forms.models.model_to_dict(self)
        data.update(dict(
            style_label=_('Style'),
            transition_effect_label=_('Transition Effect'),
            ride_label=_('Ride'),
            interval_label=_('Interval'),
            aspect_ratio_label=_('Aspect Ratio'),
        ))
        fields = [
            'style',
            'transition_effect',
            'ride',
            'interval',
            'aspect_ratio',
        ]
        if not data['ride']:
            fields.remove('interval')
        return ', '.join([
            '{key}: {value}'.format(
                key=data['{}_label'.format(field)],
                value=data[field]
            ) for field in fields
        ])

    def srcset(self):
        # more or less copied from image plugin.
        # TODO: replace with generic sizes/srcset solution
        items = collections.OrderedDict()
        if self.aspect_ratio:
            aspect_width, aspect_height = tuple([int(i) for i in self.aspect_ratio.split('x')])
        else:
            aspect_width, aspect_height = None, None
        for device in constants.DEVICES:
            width = device['width_gutter']  # TODO: should this should be based on the containing col size?
            width_tag = str(width)
            if aspect_width is not None and aspect_height is not None:
                height = int(float(width)*float(aspect_height)/float(aspect_width))
                crop = True
            else:
                height = 0
                crop = False
            items[device['identifier']] = {
                'size': (width, height),
                'size_str': "{}x{}".format(width, height),
                'width_str': "{}w".format(width),
                # 'subject_location': self.file.subject_location,
                'upscale': True,
                'crop': crop,
                'aspect_ratio': (aspect_width, aspect_height),
                'width_tag': width_tag,
            }

        return items


@python_2_unicode_compatible
class LightsliderSlidePlugin(CMSPlugin):
    excluded_attr_keys = ['class', 'href', 'target', ]
    cmsplugin_ptr = CMSPluginField()
    image = filer.fields.image.FilerImageField(
        verbose_name=_('image'),
        blank=False,
        null=True,
        related_name='+',
        on_delete=models.SET_NULL,
    )
    title = models.CharField(
        verbose_name=_('title'),
        max_length=100,
        blank=True
    )
    classes = Classes()

    def __str__(self):
        image_text = title_text = ''

        if self.image_id:
            if self.image.name:
                image_text = self.image.name
            elif self.image.original_filename \
                    and os.path.split(self.image.original_filename)[1]:
                image_text = os.path.split(self.image.original_filename)[1]
            else:
                image_text = 'Image'
        if self.title:
            text = strip_tags(self.title).strip()
            if len(text) > 100:
                title_text = '{}...'.format(text[:100])
            else:
                title_text = '{}'.format(text)

        if image_text and title_text:
            return '{} ({})'.format(image_text, title_text)
        else:
            return image_text or title_text


@python_2_unicode_compatible
class LightsliderSlideFolderPlugin(CMSPlugin):
    cmsplugin_ptr = CMSPluginField()
    folder = filer.fields.folder.FilerFolderField(
        verbose_name=_('folder'),
    )
    classes = Classes()

    def __str__(self):
        if self.folder_id:
            return self.folder.pretty_logical_path
        else:
            return _('not selected yet')



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os

from django.conf import settings
from django.utils.timezone import now
from django.utils.translation import activate, override

from aldryn_newsblog.models import Article
from cms import api

from . import NewsBlogTestCase, NewsBlogTransactionTestCase, TESTS_STATIC_ROOT

FEATURED_IMAGE_PATH = os.path.join(TESTS_STATIC_ROOT, 'featured_image.jpg')


class TestModels(NewsBlogTestCase):

    def test_create_article(self):
        article = self.create_article()
        response = self.client.get(article.get_absolute_url())
        self.assertContains(response, article.title)

    def test_delete_article(self):
        article = self.create_article()
        article_pk = article.pk
        article_url = article.get_absolute_url()
        response = self.client.get(article_url)
        self.assertContains(response, article.title)
        Article.objects.get(pk=article_pk).delete()
        response = self.client.get(article_url)
        self.assertEqual(response.status_code, 404)

    def test_auto_slugifies(self):
        activate(self.language)
        title = u'This is a title'
        author = self.create_person()
        article = Article.objects.create(
            title=title, author=author, owner=author.user,
            app_config=self.app_config, publishing_date=now(),
            is_published=True,
        )
        article.save()
        self.assertEquals(article.slug, 'this-is-a-title')
        # Now, let's try another with the same title
        article_1 = Article(
            title=title.lower(),
            author=author,
            owner=author.user,
            app_config=self.app_config,
            publishing_date=now(),
            is_published=True,
        )
        # Note, it cannot be the exact same title, else we'll fail the unique
        # constraint on the field.
        article_1.save()
        # Note that this should be "incremented" slug here.
        self.assertEquals(article_1.slug, 'this-is-a-title-1')
        article_2 = Article(
            title=title.upper(),
            author=author,
            owner=author.user,
            app_config=self.app_config,
            publishing_date=now(),
            is_published=True,
        )
        article_2.save()
        self.assertEquals(article_2.slug, 'this-is-a-title-2')

    def test_auto_existing_author(self):
        author = self.create_person()
        article = Article.objects.create(
            title=self.rand_str(), owner=author.user,
            app_config=self.app_config, publishing_date=now(),
            is_published=True,
        )
        article.save()
        self.assertEquals(article.author.user, article.owner)

        old = self.app_config.create_authors
        self.app_config.create_authors = False
        self.app_config.save()
        article = Article.objects.create(
            title=self.rand_str(), owner=author.user,
            app_config=self.app_config, publishing_date=now(),
            is_published=True,
        )
        self.app_config.create_authors = old
        self.app_config.save()
        self.assertEquals(article.author, None)

    def test_auto_new_author(self):
        user = self.create_user()
        article = Article.objects.create(
            title=self.rand_str(), owner=user,
            app_config=self.app_config, publishing_date=now(),
            is_published=True,
        )
        article.save()
        self.assertEquals(article.author.name,
                          u' '.join((user.first_name, user.last_name)))

    def test_auto_search_data(self):
        activate(self.language)

        user = self.create_user()

        lead_in = 'Hello! this text will be searchable.'

        Article.update_search_on_save = True

        article = Article.objects.create(
            title=self.rand_str(),
            owner=user,
            lead_in=lead_in,
            app_config=self.app_config,
            publishing_date=now(),
            is_published=True,
        )
        article.save()

        search_data = article.get_search_data()

        self.assertEquals(lead_in, search_data)
        self.assertEquals(article.search_data, search_data)

    def test_auto_search_data_off(self):
        activate(self.language)
        user = self.create_user()

        lead_in = 'Hello! this text will not be searchable.'

        Article.update_search_on_save = False

        article = Article.objects.create(
            title=self.rand_str(),
            owner=user,
            lead_in=lead_in,
            app_config=self.app_config,
            publishing_date=now(),
            is_published=True,
        )
        article.save()

        search_data = article.get_search_data()

        # set it back to true
        Article.update_search_on_save = True

        self.assertEquals(lead_in, search_data)
        self.assertNotEquals(article.search_data, search_data)

    def test_has_content(self):
        # Just make sure we have a known language
        activate(self.language)
        title = self.rand_str()
        content = self.rand_str()
        author = self.create_person()
        article = Article.objects.create(
            title=title, slug=self.rand_str(), author=author, owner=author.user,
            app_config=self.app_config, publishing_date=now(),
            is_published=True,
        )
        article.save()
        api.add_plugin(article.content, 'TextPlugin', self.language)
        plugin = article.content.get_plugins()[0].get_plugin_instance()[0]
        plugin.body = content
        plugin.save()
        response = self.client.get(article.get_absolute_url())
        self.assertContains(response, title)
        self.assertContains(response, content)

    def test_change_title(self):
        """
        Test that we can change the title of an existing, published article
        without issue. Also ensure that the slug does NOT change when changing
        the title alone.
        """
        activate(self.language)
        initial_title = "This is the initial title"
        initial_slug = "this-is-the-initial-title"
        author = self.create_person()
        article = Article.objects.create(
            title=initial_title, author=author, owner=author.user,
            app_config=self.app_config, publishing_date=now(),
            is_published=True,
        )
        article.save()
        self.assertEquals(article.title, initial_title)
        self.assertEquals(article.slug, initial_slug)
        # Now, let's try to change the title
        new_title = "This is the new title"
        article.title = new_title
        article.save()
        article = self.reload(article)
        self.assertEquals(article.title, new_title)
        self.assertEquals(article.slug, initial_slug)


class TestModelsTransactions(NewsBlogTransactionTestCase):

    def test_duplicate_title_and_language(self):
        """
        Test that if user attempts to create an article with the same name and
        in the same language as another, it will not raise exceptions.
        """
        title = "Sample Article"
        author = self.create_person()
        original_lang = settings.LANGUAGES[0][0]
        # Create an initial article in the first language
        article1 = Article(
            title=title, author=author, owner=author.user,
            app_config=self.app_config, publishing_date=now(),
            is_published=True,
        )
        article1.set_current_language(original_lang)
        article1.save()

        # Now try to create an article with the same title in every possible
        # language and every possible language contexts.
        for context_lang, _ in settings.LANGUAGES:
            with override(context_lang):
                for article_lang, _ in settings.LANGUAGES:
                    try:
                        article = Article(
                            author=author, owner=author.user,
                            app_config=self.app_config, publishing_date=now(),
                            is_published=True,
                        )
                        article.set_current_language(article_lang)
                        article.title = title
                        article.save()
                    except Exception:
                        self.fail('Creating article in process context "{0}" '
                            'and article language "{1}" with identical name '
                            'as another "{2}" article raises exception'.format(
                                context_lang,
                                article_lang,
                                original_lang,
                            ))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_lightslider', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='lightsliderplugin',
            name='transition_style',
            field=models.CharField(default=b'', max_length=50, verbose_name='Transition Style', blank=True, choices=[(b'slide', 'Slide'), (b'fade', 'Fade')]),
        ),
        migrations.AlterField(
            model_name='lightsliderplugin',
            name='transition_effect',
            field=models.CharField(default=b'', max_length=50, verbose_name='Transition Effect', blank=True, choices=[(b'linear', 'Linear'), (b'ease', 'Ease'), (b'ease-in', 'Ease in'), (b'ease-out', 'Ease out'), (b'ease-in-out', 'Ease in out')]),
        ),
    ]



import os
from setuptools import find_packages, setup
import aldryn_lightslider

version = aldryn_lightslider.__version__
here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='aldryn-lightslider',
    version=version,
    packages=find_packages(),
    include_package_data=True,
    install_requires=(
        'django-filer',
        'django-cms>=3.0',
        'djangocms-admin-style',
        'cmsplugin-filer',
    ),
    # install_requires=[i.strip() for i in open(os.path.join(here,"requirements.txt")).readlines()],
    license='GNU License', 
    description='LightSlider aldryn addon and djangocms plugin.',
    long_description=README,
    url='https://github.com/ninjabit/aldryn-lightslider',
    author='Tobia Ghiraldini',
    author_email='tobia.ghiraldini@ninjabit.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext_lazy as _

from . import models, forms

###############
# Lightslider #
###############


# Base Classes
class LightsliderBase(CMSPluginBase):
    module = _("Bootstrap3")


class LightsliderSlideBase(LightsliderBase):
    require_parent = True
    parent_classes = ['LightsliderCMSPlugin']

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        context['parent'] = instance.parent.get_plugin_instance()[0]
        context['image'] = instance.image
        return context

    def get_slide_template(self, instance, name='slide'):
        if instance.parent is None:
            # defaults to default style
            style = models.LightsliderPlugin.STYLE_DEFAULT
        else:
            style = getattr(instance.parent.get_plugin_instance()[0], 'style', models.LightsliderPlugin.STYLE_DEFAULT)
        return 'aldryn_lightslider/plugins/lightslider/{}/{}.html'.format(style, name)

    def get_render_template(self, context, instance, placeholder):
        return self.get_slide_template(instance=instance)


# Plugins
class LightsliderCMSPlugin(LightsliderBase):
    name = _('Lightslider')
    model = models.LightsliderPlugin
    change_form_template = 'admin/aldryn_lightslider/base.html'
    render_template = False
    form = forms.LightsliderPluginForm
    allow_children = True
    child_classes = [
        'LightsliderSlideCMSPlugin',
        # 'Bootstrap3CarouselSlideFolderCMSPlugin',
    ]
    fieldsets = (
        (None, {
            'fields': (
                'number_of_concurrent_slides',
                'number_of_paginating_slides',
                'style',
                'transition_style',
                'transition_effect',
                ('ride', 'interval'),
                'aspect_ratio',

            )
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': (
                'classes',
                'pause',
                'wrap',
            ),
        }),
    )

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        if instance.child_plugin_instances:
            number_of_slides = sum([
                plugin.folder.file_count
                if isinstance(plugin, LightsliderCMSPlugin) else 1
                for plugin in instance.child_plugin_instances
            ])
        else:
            number_of_slides = 0
        context['slides'] = range(number_of_slides)
        return context

    def get_render_template(self, context, instance, placeholder):
        return 'aldryn_lightslider/plugins/lightslider/{}/lightslider.html'.format(
            instance.style)


class LightsliderSlideCMSPlugin(LightsliderSlideBase):
    form = forms.LightsliderSlidePluginForm
    model = models.LightsliderSlidePlugin
    name = _('Lightslider Slide')
    change_form_template = 'admin/aldryn_lightslider/base.html'
    allow_children = True
    fieldsets = (
        (None, {
            'fields': (
                'image',
                'title',
            )
        }),
    )


class LightsliderSlideFolderCMSPlugin(LightsliderSlideBase):
    """
    Slide Plugin that renders a slide for each image in the linked folder.
    """
    name = _('Lightslider Slides Folder')
    model = models.LightsliderSlideFolderPlugin

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        context['slide_template'] = self.get_slide_template(
            instance=instance,
            name='image_slide',
        )
        return context

    def get_render_template(self, context, instance, placeholder):
        return self.get_slide_template(instance=instance, name='slide_folder')


plugin_pool.register_plugin(LightsliderCMSPlugin)
plugin_pool.register_plugin(LightsliderSlideCMSPlugin)
# plugin_pool.register_plugin(Bootstrap3CarouselSlideFolderCMSPlugin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import filer.fields.folder
import django.db.models.deletion
import filer.fields.image
import aldryn_lightslider.models


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('filer', '0006_auto_20160623_1627'),
    ]

    operations = [
        migrations.CreateModel(
            name='LightsliderPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('number_of_concurrent_slides', models.PositiveIntegerField(default=3, help_text='How many images displayed at the same time.', verbose_name='Number of concurrent slides')),
                ('number_of_paginating_slides', models.PositiveIntegerField(default=1, help_text='How many images to slide out when going forward or backward.', verbose_name='Number of paginating slides')),
                ('style', models.CharField(default=b'standard', max_length=50, verbose_name='Style', choices=[(b'standard', 'Standard')])),
                ('aspect_ratio', models.CharField(default=b'', max_length=10, verbose_name='aspect ratio', blank=True, choices=[('1x1', '1x1'), ('4x3', '4x3'), ('16x9', '16x9'), ('16x10', '16x10'), ('21x9', '21x9'), ('3x4', '3x4'), ('9x16', '9x16'), ('10x16', '10x16'), ('9x21', '9x21')])),
                ('transition_effect', models.CharField(default=b'', max_length=50, verbose_name='Transition Effect', blank=True, choices=[(b'slide', 'Slide')])),
                ('ride', models.BooleanField(default=True, help_text='Whether to mark the carousel as animating starting at page load.', verbose_name='Ride')),
                ('interval', models.IntegerField(default=5000, help_text='The amount of time to delay between automatically cycling an item.', verbose_name='Interval')),
                ('wrap', models.BooleanField(default=True, help_text='Whether the carousel should cycle continuously or have hard stops.')),
                ('pause', models.BooleanField(default=True, help_text='Pauses the cycling of the carousel on mouseenter and resumes the cycling of the carousel on mouseleave.')),
                ('classes', aldryn_lightslider.models.Classes(default=b'', help_text=b'space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='LightsliderSlideFolderPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('classes', aldryn_lightslider.models.Classes(default=b'', help_text=b'space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>', blank=True)),
                ('folder', filer.fields.folder.FilerFolderField(verbose_name='folder', to='filer.Folder')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='LightsliderSlidePlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, related_name='+', primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('title', models.CharField(max_length=100, verbose_name='title', blank=True)),
                ('classes', aldryn_lightslider.models.Classes(default=b'', help_text=b'space separated classes that are added to the class. see <a href="http://getbootstrap.com/css/" target="_blank">bootstrap docs</a>', blank=True)),
                ('image', filer.fields.image.FilerImageField(related_name='+', on_delete=django.db.models.deletion.SET_NULL, verbose_name='image', to='filer.Image', null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import NoReverseMatch, reverse

from unittest import TestCase

from ..utils import add_prefix_to_path, default_reverse


class TestAddPrefixToPath(TestCase):

    def test_no_directory(self):
        self.assertEqual(
            add_prefix_to_path('template.html', 'prefix'),
            'prefix/template.html')

    def test_with_directory(self):
        self.assertEqual(
            add_prefix_to_path('directory/template.html', 'prefix'),
            'directory/prefix/template.html')

    def test_default_reverse(self):
        non_pattern = 'invalid_url_pattern'

        # *********************************************************************
        # NOTE: Until we drop support for Python 2.6, we can't use the context-
        # manager form of assertRaises =(
        # *********************************************************************

        # First, test that the normal reverse works as expected (this proves
        # that our setup conditions make sense)
        try:
            reverse(non_pattern)
        except NoReverseMatch:
            pass
        except Exception as e:
            raise e
        else:
            self.fail('reverse did not raise expected NoReverseMatch.')

        # Test that default_reverse without a default set, acts the same
        # as above.
        try:
            default_reverse(non_pattern)
        except NoReverseMatch:
            pass
        except Exception as e:
            raise e
        else:
            self.fail('reverse did not raise expected NoReverseMatch.')

        # Now prove that default_reverse WITH a default returns that default.
        # Trying with multiple types of values
        class MyClass():
            pass

        defaults = [
            'some_value', '', 'Undefined',
            1, 0,
            True, False,
            None, MyClass, MyClass(),
        ]

        for default in defaults:
            try:
                self.assertEquals(
                    default_reverse(non_pattern, default=default),
                    default
                )
            except:
                self.fail('default_reverse raised exception even though we '
                          'set a default value of: {0}.'.format(default))
